"""Compatibilidad retrocompatible para :mod:`cobra.transpilers`.

El contenido real se encuentra en ``cobra.transpilers.common.utils``.
"""

from cobra.transpilers.common.utils import BaseTranspiler, save_file

__all__ = ["BaseTranspiler", "save_file"]
